@@ -67,6 +67,7 @@ class RubyLexer
 
   # :startdoc:
 
+  attr_accessor :lineno # we're bypassing oedipus' lineno handling.
   attr_accessor :brace_nest
   attr_accessor :cmdarg
   attr_accessor :command_start
@@ -270,7 +271,7 @@ class RubyLexer
   end
 
   def is_beg?
-    in_lex_state? :expr_beg, :expr_value, :expr_mid, :expr_class
+    in_lex_state? :expr_beg, :expr_value, :expr_mid, :expr_class, :expr_labelarg
   end
 
   def is_end?
@@ -321,6 +322,7 @@ class RubyLexer
     end
 
     @comments << matched
+    self.lineno += matched.count("\n")
 
     nil # TODO
   end
@@ -404,7 +406,8 @@ class RubyLexer
   end
 
   def process_gvar_oddity text
-    result :expr_end, "$", "$" # TODO: wtf is this?
+    return result :expr_end, "$", "$" if text == "$" # TODO: wtf is this?
+    rb_compile_error "#{text.inspect} is not allowed as a global variable name"
   end
 
   def process_ivar text
@@ -671,7 +674,7 @@ class RubyLexer
       end
 
     if !ruby18 and is_label_possible? and scan(/:(?!:)/) then
-      return result(:expr_beg, :tLABEL, [token, self.lineno])
+      return result(:expr_labelarg, :tLABEL, [token, self.lineno])
     end
 
     unless in_lex_state? :expr_dot then
@@ -1142,7 +1145,12 @@ class RubyLexer
 
     if expand
       case
-      when scan(/#(?=[$@])/) then
+      when scan(/#(?=\$(-.|[a-zA-Z_0-9~\*\$\?!@\/\\;,\.=:<>\"\&\`\'+]))/) then
+        # TODO: !ISASCII
+        # ?! see parser_peek_variable_name
+        return :tSTRING_DVAR, nil
+      when scan(/#(?=\@\@?[a-zA-Z_])/) then
+        # TODO: !ISASCII
         return :tSTRING_DVAR, nil
       when scan(/#[{]/) then
         return :tSTRING_DBEG, nil
