@@ -79,7 +79,7 @@ class RPStringScanner < StringScanner
     alias :old_scan :scan
     def scan re
       s = old_scan re
-      where = caller.first.split(/:/).first(2).join(":")
+      where = caller[1].split(/:/).first(2).join(":")
       d :scan => [s, where] if s
       s
     end
@@ -91,7 +91,7 @@ class RPStringScanner < StringScanner
 end
 
 module RubyParserStuff
-  VERSION = "3.6.2" unless constants.include? "VERSION" # SIGH
+  VERSION = "3.6.6" unless constants.include? "VERSION" # SIGH
 
   attr_accessor :lexer, :in_def, :in_single, :file
   attr_reader :env, :comments
