@@ -1,4 +1,3 @@
-#!/usr/local/bin/ruby
 # encoding: US-ASCII
 
 require 'rubygems'
@@ -299,7 +298,7 @@ class TestRubyLexer < Minitest::Test
     assert_lex3("{a:",
                 nil,
                 :tLBRACE, "{", :expr_beg,
-                :tLABEL,  "a", :expr_beg)
+                :tLABEL,  "a", :expr_labelarg)
   end
 
   def test_yylex_label_in_params__19
@@ -309,7 +308,7 @@ class TestRubyLexer < Minitest::Test
                 nil,
                 :tIDENTIFIER, "foo", :expr_cmdarg,
                 :tLPAREN2,    "(",   :expr_beg,
-                :tLABEL,      "a",   :expr_beg)
+                :tLABEL,      "a",   :expr_labelarg)
   end
 
   def test_yylex_paren_string_parens_interpolated
@@ -853,9 +852,13 @@ class TestRubyLexer < Minitest::Test
     end
   end
 
-  def test_yylex_dollar
-    assert_lex3("$", nil, "$", "$", :expr_end)
-    # FIX: wtf is this?!?
+  def test_yylex_dollar_bad
+    e = refute_lex("$%")
+    assert_includes(e.message, "is not allowed as a global variable name")
+  end
+
+  def test_yylex_dollar_eos
+    assert_lex3("$", nil, "$", "$", :expr_end) # FIX: wtf is this?!?
   end
 
   def test_yylex_dot # HINT message sends
@@ -2290,6 +2293,14 @@ class TestRubyLexer < Minitest::Test
                 :tSTRING_END,     "\"",         :expr_end)
   end
 
+  def test_yylex_string_double_pound_dollar_bad
+    assert_lex3('"#$%"', nil,
+
+                :tSTRING_BEG,     "\"",  :expr_beg,
+                :tSTRING_CONTENT, "#$%", :expr_beg,
+                :tSTRING_END,     "\"",  :expr_end)
+  end
+
   def test_yylex_string_double_nested_curlies
     assert_lex3("%{nest{one{two}one}nest}",
                 nil,
@@ -2612,6 +2623,35 @@ class TestRubyLexer < Minitest::Test
     end
   end
 
+  def test_yylex_sym_quoted
+    assert_lex(":'a'",
+               s(:lit, :a),
+
+               :tSYMBOL, "a", :expr_end, 0, 0)
+  end
+
+  def test_yylex_hash_colon
+    assert_lex("{a:1}",
+               s(:hash, s(:lit, :a), s(:lit, 1)),
+
+               :tLBRACE, "{", :expr_beg,      0, 1,
+               :tLABEL,  "a", :expr_labelarg, 0, 1,
+               :tINTEGER, 1,  :expr_end,      0, 1,
+               :tRCURLY, "}", :expr_endarg,   0, 0)
+  end
+
+  def test_yylex_hash_colon_quoted_22
+    skip "Waiting for 2.2 parser"
+
+    assert_lex("{'a':1}",
+               s(:hash, s(:lit, :a), s(:lit, 1)),
+
+               :tLBRACE, "{", :expr_beg,    0, 1,
+               :tSYMBOL, "a", :expr_end,    0, 1,
+               :tINTEGER, 1,  :expr_end,    0, 1,
+               :tRCURLY, "}", :expr_endarg, 0, 0)
+  end
+
   def test_ruby21_new_numbers
     skip "Don't have imaginary and rational literal lexing yet"
 
