@@ -1,4 +1,3 @@
-#!/usr/local/bin/ruby
 # encoding: utf-8
 
 # ENV['VERBOSE'] = "1"
@@ -317,6 +316,15 @@ module TestRubyParserShared
     assert_parse rb, pt
   end
 
+  def test_eq_begin_line_numbers
+    rb = "1\n=begin\ncomment\ncomment\n=end\n2"
+    pt = s(:block,
+           s(:lit, 1).line(1),
+           s(:lit, 2).line(6))
+
+    assert_parse rb, pt
+  end
+
   def test_bug_call_arglist_parens
     rb = 'g ( 1), 2'
     pt = s(:call, nil, :g, s(:lit, 1), s(:lit, 2))
@@ -758,6 +766,15 @@ module TestRubyParserShared
     assert_parse rb, pt
   end
 
+  def test_parse_line_multiline_str_literal_n
+    rb = "\"a\\nb\"\n1"
+    pt = s(:block,
+           s(:str, "a\nb").line(1),
+           s(:lit, 1).line(2)).line(1)
+
+    assert_parse rb, pt
+  end
+
   def test_parse_line_iter_call_parens
     rb = "f(a) do |x, y|\n  x + y\nend"
 
@@ -2125,21 +2142,30 @@ module TestRubyParserShared20to21
   def test_defn_kwarg_env
     rb = "def test(**testing) test_splat(**testing) end"
     pt = s(:defn, :test, s(:args, :"**testing"),
-           s(:call, nil, :test_splat, s(:kwsplat, s(:lvar, :testing))))
+           s(:call, nil, :test_splat, s(:hash, s(:kwsplat, s(:lvar, :testing)))))
 
     assert_parse rb, pt
   end
 
   def test_call_arg_kwsplat
     rb = "a(b, **1)"
-    pt = s(:call, nil, :a, s(:call, nil, :b), s(:kwsplat, s(:lit, 1)))
+    pt = s(:call, nil, :a, s(:call, nil, :b), s(:hash, s(:kwsplat, s(:lit, 1))))
+
+    assert_parse rb, pt
+  end
+
+  def test_call_arg_assoc_kwsplat
+    rb = "f(1, kw: 2, **3)"
+    pt = s(:call, nil, :f,
+           s(:lit, 1),
+           s(:hash, s(:lit, :kw), s(:lit, 2), s(:kwsplat, s(:lit, 3))))
 
     assert_parse rb, pt
   end
 
   def test_call_kwsplat
     rb = "a(**1)"
-    pt = s(:call, nil, :a, s(:kwsplat, s(:lit, 1)))
+    pt = s(:call, nil, :a, s(:hash, s(:kwsplat, s(:lit, 1))))
 
     assert_parse rb, pt
   end
@@ -3211,16 +3237,55 @@ class TestRuby21Parser < RubyParserTestCase
   end
 
   def test_block_kw
+    rb = "blah { |k:42| }"
+    pt = s(:iter, s(:call, nil, :blah), s(:args, s(:kwarg, :k, s(:lit, 42))))
+
+    assert_parse rb, pt
+
+    rb = "blah { |k:42| }"
+    assert_parse rb, pt
+  end
+
+  def test_block_kw__required
+    rb = "blah do |k:| end"
+    pt = s(:iter, s(:call, nil, :blah), s(:args, s(:kwarg, :k)))
+
+    assert_parse rb, pt
+
+    rb = "blah do |k:| end"
+    assert_parse rb, pt
+  end
+
+  def test_stabby_block_kw
     rb = "-> (k:42) { }"
     pt = s(:iter, s(:call, nil, :lambda), s(:args, s(:kwarg, :k, s(:lit, 42))))
 
     assert_parse rb, pt
   end
 
-  def test_block_kw__required
+  def test_stabby_block_kw__required
     rb = "-> (k:) { }"
     pt = s(:iter, s(:call, nil, :lambda), s(:args, s(:kwarg, :k)))
 
     assert_parse rb, pt
   end
+
+  def test_parse_line_heredoc_hardnewline
+    skip "not yet"
+
+    rb = <<-'CODE'.gsub(/^      /, '')
+      <<-EOFOO
+      \n\n\n\n\n\n\n\n\n
+      EOFOO
+
+      class Foo
+      end
+    CODE
+
+    pt = s(:block,
+           s(:str, "\n\n\n\n\n\n\n\n\n\n").line(1),
+           s(:class, :Foo, nil).line(5)).line(1)
+
+    assert_parse rb, pt
+  end
 end
